/*
	netLib.c - Module file for netLib Network Communication Library
	v0.1 - 2007.02.27
	Coded by Ian Seyler (iseyler@gmail.com)
	http://bubbai.ath.cx/projects/netlib/
*/


#include "netLib.h"


/*========================================================*
 * Function : netInit()
 * Purpose : Initiate the netLib library.
 * Entry   : nothing
 * Returns : 0 on success
 *========================================================*/

int netInit ()
{
	printf("\nnetLib: Start");

	#ifdef MS_WINDOWS
	{
		printf(" Windows networking");
		WSADATA wsaData; // If WSADATA doesn't work then try WSAData instead.
		if (WSAStartup(MAKEWORD(1, 1), &wsaData) != 0) // Init the WinSock library.
		{
			printf(", WSAStartup Error!\n");
			exit(1);
		}
	}
	#endif
	#ifdef UNIX
		printf(" Unix networking");
	#endif
	#ifdef BEOS
		printf(" BeOS networking");
	#endif
	printf(" (v0.1)"); // The version of netLib
	return 0;
}


/*========================================================*
 * Function : netStop()
 * Purpose : Stop the netLib library.
 * Entry   : nothing
 * Returns : 0 on success
 *========================================================*/

int netStop ()
{
	printf("\nnetLib: Stop");

	#ifdef MS_WINDOWS
	{
		WSACleanup(); // We are done using the WinSock library.
	}
	#endif
	return 0;
}


/*========================================================*
 * Function : netInitSocket()
 * Purpose : Initiate a new netSocket.
 * Entry   : nothing
 * Returns : returns a pointer to a netSocket, NULL on error
 *========================================================*/

netSocket* netInitSocket ()
{
	netSocket *netSock;

	printf("\nnetLib: InitSocket");

	netSock = malloc(sizeof(netSocket));
	if (netSock == NULL)
	{
		printf(", Error!");
		return NULL;
	}
	else
	{
		memset(netSock, '\0', sizeof(netSocket)); // zero the entire structure
		return netSock;
	}
}


/*========================================================*
 * Function : netInitBuffer()
 * Purpose : Initiate a new netBuffer.
 * Entry   : int size : The size in bytes for the buffer
 * Returns : returns a pointer to a netBuffer, NULL on error
 *========================================================*/

netBuffer* netInitBuffer (int size)
{
	netBuffer *netBuff;

	printf("\nnetLib: InitBuffer");

	netBuff = malloc(sizeof(netBuffer));
	if (netBuff == NULL)
	{
		printf(", Error! (Allocating Pointer Buffer)");
		return NULL;
	}
	else
	{
		memset(netBuff, '\0', sizeof(netBuffer)); // zero the entire structure
		netBuff->data = malloc(size);
		if (netBuff->data == NULL)
		{
			printf(", Error! (Allocating Data Buffer)");
			free(netBuff); // We couln't get enough memory for the data buffer so free what we have used so far.
			return NULL;		
		}
		else
		{
			netBuff->size = size; // set the size for buffer data
			memset(netBuff->data, '\0', size); // zero the data structure
			return netBuff;
		}
	}
}


/*========================================================*
 * Function : netConnect()
 * Purpose : Connect a netSocket to the specified destination.
 * Entry   : netSocket *netSock = Socket to connect
 *           char *address = Character string of the destination (eg: "bubbai.ath.cx" or "192.168.0.1")
 *           unsigned short port = The port on the destination to connect to
 * Returns : 0 on success, anything else on failure
 *========================================================*/

int netConnect (netSocket *netSock, char *address, unsigned short port)
{
	int tint = 0, socketargs = 0;
	struct hostent *h;

	printf("\nnetLib: Connect");

	printf(" %s:%d", address, port);
	memset(netSock, '\0', sizeof(netSocket)); // zero the entire structure

	h = gethostbyname(address); // convert host name to IP and hold it in hostent structure
	if (h == NULL)
	{
		printf(", name lookup failed");
		return -1; // DNS has failed.. bail out.
	}

	address = inet_ntoa(*((struct in_addr *)h->h_addr)); // get the IP address out of the hostent sturcture
	netSock->addr.sin_family = AF_INET;
	netSock->addr.sin_port = htons(port); // setup the port we will be connecting to.
	netSock->addr.sin_addr.s_addr = inet_addr(address); // Translate the IP address string to a usable address. Does work in Windows
	netSock->number = socket(PF_INET, SOCK_STREAM, socketargs);
	if (netSock->number == -1)
	{
		printf("socket ERROR");
		return -1;
	}

	tint = connect(netSock->number, (struct sockaddr*)&netSock->addr, sizeof(netSock->addr)); // connect the socket

//	#ifdef BEOS_BONE	
//		printf(", BeOS BONE Debug: %d", tint); // Issues here.. connect() always returning 0 on BONE
//	#endif

	if (tint != 0)
	{
		printf(", FAILED");
		netDisconnect (netSock); // properly close the socket
		memset(netSock, '\0', sizeof(netSocket)); // zero the entire structure
	}
	else
	{
		printf(", OK");
		// Should protect against SIGPIPE here. Check code from netAccept.
		netSock->inuse = 1; // mark this socket as being in use since it was succesful
	}
	return tint;
}


/*========================================================*
 * Function : netDisconnect()
 * Purpose : Disconnect a netSocket.
 * Entry   : netSocket *netSock = The socket to disconnect
 * Returns : 0 on success
 *========================================================*/

int netDisconnect (netSocket *netSock)
{
	printf("\nnetLib: Disconnect");

	#ifdef UNIX // Unix uses close(), Windows and BeOS use closesocket().
	{
		close(netSock->number);
	}
	#else
	{
		closesocket(netSock->number);
	}
	#endif
	netSock->inuse = 0; // Mark the socket as no longer being used, it could be memset as well.
	return 0;
}


/*========================================================*
 * Function : netListen()
 * Purpose : Listen on a specified netSocket.
 * Entry   : netSocket *netSock = Socket to listen on
 *           unsigned short port = Port number to listen on
 * Returns : 0 on success, -1 on failure
 *========================================================*/

int netListen (netSocket *netSock, unsigned short port)
{
	int tint = 0, socketargs = 0;
	#ifdef MS_WINDOWS // Windows wants a char, everyone else wants an int.
		char *setsockchar;
	#else
		int setsockint = 1;
	#endif

	printf("\nnetLib: Listen");

	memset(netSock, '\0', sizeof(netSocket)); // zero the entire structure
	netSock->addr.sin_family = AF_INET;
	netSock->addr.sin_port = htons(port); // The clients will connect to this port
	netSock->addr.sin_addr.s_addr = INADDR_ANY; // autoselect IP address
	netSock->number = socket(PF_INET, SOCK_STREAM, socketargs);
	if (netSock->number == -1)
	{
		printf("socket ERROR");
		return -1;
	}

	#ifdef MS_WINDOWS
		if (setsockopt(netSock->number, SOL_SOCKET, SO_REUSEADDR, setsockchar, sizeof(int)) == -1)
	#else
		if (setsockopt(netSock->number, SOL_SOCKET, SO_REUSEADDR, &setsockint, sizeof(int)) == -1)
	#endif
			printf(", setsockopt Warning");

	tint = bind(netSock->number, (struct sockaddr*)&netSock->addr, sizeof(struct sockaddr));
	if (tint == 0)
		printf(", Bind Sucess");
	else
	{
		printf(", Bind Failure");
		netDisconnect(netSock);
		return -1;
	}

	tint = listen(netSock->number, 10); // set up to be a server (listening) socket, 2nd argument is backlog
	if (tint == 0)
		printf(", Listen Sucess");
	else
	{
		printf(", Listen Failure");
		netDisconnect(netSock);
		return -1;
	}

	return 0;
}


/*========================================================*
 * Function : netAccept()
 * Purpose : Accept a connection from a listening port.
 * Entry   : netSocket *listenSock = The listening socket
 *           netSocket *netSock = The netSocket that has just connected
 * Returns : 0 on success, -1 on failure
 *========================================================*/

int netAccept (netSocket *listenSock, netSocket *netSock)
{
	socklen_t meh;
	int setsockint = 1;

	printf("\nnetLib: Accept");

	memset(netSock, '\0', sizeof(netSocket)); // zero the entire structure
	netSock->number = accept(listenSock->number, (struct sockaddr*)&netSock->addr, &meh);
	if (netSock->number == -1)
	{
		printf(", Error");
		return -1;
	}
	else
	{
		printf(", Got a connecting socket from %s",inet_ntoa(netSock->addr.sin_addr));
		#ifdef SO_NOSIGPIPE
		{
			// Stops "Broken pipe" errors in Unix. SIGPIPE will stop the program from running. We just want the error number.
			if (setsockopt(netSock->number, SOL_SOCKET, SO_NOSIGPIPE, &setsockint, sizeof(int)) == -1)
			{
				printf(", setsockopt Error");
				netDisconnect(netSock);
				return -1;
			}
		}
		#endif
		netSock->inuse = 1; // mark the socket as being in use
		return 0;
	}
}


/*========================================================*
 * Function : netSelect()
 * Purpose : Checks if a netSocket is waiting for action.
 * Entry   : netSocket *netSock = The socket to check
 *           int timeout = The timeout value (in usec)
 * Returns : 0 on no action, 1 on action needed, -1 on error
 *========================================================*/

int netSelect (netSocket *netSock, int timeout) //netIsSocketWaiting
{
	fd_set selectlist;
	struct timeval tv;
	int selreturn;

	printf("\nnetLib: Select");

	FD_ZERO(&selectlist);	
	FD_SET(netSock->number, &selectlist);

	// Set the timeout delay in usecs
	tv.tv_sec = 0;
	tv.tv_usec = timeout;

	// Run the actual select(), only checking the one socket that needs reading here.
	selreturn = select(netSock->number+1, &selectlist, NULL, NULL, &tv);
	// Far from being the best way to use select. Using this call on each socket instead of a list of sockets will be slow however the netSelect function was designed for its simplicity.

	if (selreturn == 0) // This socket is not waiting on anything
	{
		return 0;
	}
	else if (selreturn > 0) // There is something to work on
	{
		return 1;	
	}
	else // select returned an error
	{
		printf(", ERROR");
		return -1;	
	}
}


/*========================================================*
 * Function : netSend()
 * Purpose : Send a number of bytes to a netSocket.
 * Entry   : netSocket *netSock = Socket to send data to
 *           char *data = Pointer to a character string, this is the data that will be transfered
 *           int bytes = the number of bytes to send starting at *data
 * Returns : number of bytes that were sent
 *           -1 on error, 0 on closed connection
 *========================================================*/

int netSend (netSocket *netSock, char *data, int bytes)
{
	int byte_count = 0;

	printf("\nnetLib: Send");

	byte_count = send(netSock->number, data, bytes, 0);
	printf(", transfered %d bytes", byte_count);
	if (byte_count == -1)
	{
		printf (", Connection closed with error");
		netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
		return -1;
	}
	else if (byte_count == 0)
	{
		printf (", Connection closed");
		netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
		return 0;
	}

	return byte_count;
}


/*========================================================*
 * Function : netRecv()
 * Purpose : Recveive a number of bytes from a netSocket.
 * Entry   : netSocket *netSock = Socket to receive data from
 *           char *data = Pointer to a character string, this is where the data will be stored
 *           int bytes = the number of bytes to receive
 * Returns : number of bytes that were received
 *           -1 on error, 0 on closed connection
 *========================================================*/

int netRecv (netSocket *netSock, char *data, int bytes)
{
	int byte_count = 0;

	printf("\nnetLib: Receive");

	byte_count = recv(netSock->number, data, bytes, 0);
	printf(", transfered %d bytes", byte_count);
	if (byte_count == -1)
	{
		printf (", Connection closed with error");
		netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
		return -1;
	}
	else if (byte_count == 0)
	{
		printf (", Connection closed");
		netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
		return 0;
	}

	return byte_count;
}


/*========================================================*
 * Function : netSendBuffer()
 * Purpose : Send data from a netBuffer to a netSocket.
 * Entry   : netSocket *netSock = The socket to send to
 *           netBuffer *netBuff = The buffer to be sent out
 * Returns : number of bytes that were sent
 *           -1 on error, 0 on closed connection
 *========================================================*/

int netSendBuffer (netSocket *netSock, netBuffer *netBuff)
{
	int byte_count = 0;
	int bytes_sent = 0; // keep track of how many bytes have been sent in this run
	int bytes_left = netBuff->size - netBuff->at;

	printf("\nnetLib: Send Buffer");

	while(bytes_sent != netBuff->size) // keep going until the buffer has been sent
	{
		byte_count = send(netSock->number, netBuff->data+bytes_sent, bytes_left, 0);
		printf(", transfered %d bytes", byte_count);
		if (byte_count == -1)
		{
			printf (", Connection closed with error");
			netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
			return -1;
		}
		else if (byte_count == 0)
		{
			printf (", Connection closed");
			netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
			return 0;
		}
		else
		{
			bytes_sent += byte_count;
			netBuff->at += byte_count; // keep track of where we are in the buffer
			bytes_left -= byte_count;
			if (bytes_sent < netBuff->size)
				printf(", waiting to send %d bytes", bytes_left);
		}
	}

	if (netBuff->at == netBuff->size) // We got everything we need.. reset at
		netBuff->at = 0;

	return bytes_sent;
}


/*========================================================*
 * Function : netRecvBuffer()
 * Purpose : Receive data from a netSocket to a netBuffer.
 * Entry   : netSocket *netSock = The socket to receive from
 *           netBuffer *netBuff = The buffer where the incoming data will be stored
 * Returns : number of bytes that were received
 *           -1 on error, 0 on closed connection
 *========================================================*/

int netRecvBuffer (netSocket *netSock, netBuffer *netBuff)
{
	int byte_count = 0;
	int bytes_recv = 0; // keep track of how many bytes have been received in this run
	int bytes_left = netBuff->size - netBuff->at;

	printf("\nnetLib: Receive Buffer");

	while(bytes_recv != netBuff->size) // keep going until the buffer is full
	{
		byte_count = recv(netSock->number, netBuff->data+bytes_recv, bytes_left, 0);
		printf(", transfered %d bytes", byte_count);
		if (byte_count == -1)
		{
			printf (", Connection closed with error");
			netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
			return -1;
		}
		else if (byte_count == 0)
		{
			printf (", Connection closed");
			netDisconnect(netSock); // Since the connect has closed we can disconnect it, not sure this should be done here
			return 0;
		}
		else
		{
			bytes_recv += byte_count;
			netBuff->at += byte_count; // keep track of where we are in the buffer
			bytes_left -= byte_count;
			if (bytes_recv < netBuff->size)
				printf(", waiting to receive %d bytes", bytes_left);
		}
	}

	if (netBuff->at == netBuff->size) // We got everything we need.. reset at
		netBuff->at = 0;

	return bytes_recv;
}
